@@ -1,4 +1,4 @@
-/* $Id: config.h,v 1.216 2005/10/12 03:30:05 jared Exp $ */
+/* $Id: config.h,v 1.232 2014/07/09 16:32:35 jared Exp $ */
 
 #include "defines.h"
 
@@ -88,7 +88,7 @@
 #endif /* HAVE_UCD_SNMP_VERSION_H */
 
 
-#define SYSM_VERS	"v0.92.2"
+#define SYSM_VERS	"v0.93"
 #ifdef _PATH_VARRUN
 #define PIDFILE		_PATH_VARRUN "sysmond.pid"
 #else
@@ -290,6 +290,8 @@ struct hostinfo {
 	unsigned char *url; /* url for www */
 	unsigned char *url_text; /* text to find within url */
 	unsigned char *command; /* command to run on failure */
+	unsigned char *group; /* group of object */
+	unsigned char *notes; /* user attached notes for object */
 	unsigned long totalchecked; /* total times checked */
 	unsigned long totaldown; /* total times checked as down */
 	unsigned long downct; /* number of times counted as down.. */
@@ -306,13 +308,14 @@ struct hostinfo {
 	bool reverse; /* if true then {if down, follow siblings}, else
 			behave as we do otherwise */
 	bool contacted; /* true if mailed contact -- false if not */
+	bool acked; /* true if someone has acked the alert */
 	time_t lastcontacted; /* last time we "contacted" someone */
 
 	int contact_when; /* page someone when it comes up */
 	
 	bool queued; /* 0 if not, 1 if in queue */
 	bool warnlog; /* 0 if already done warnlog, 1 if not */
-	bool trace;
+	bool trace; /* 1 if object should be have debugging enabled for it */
 
 	time_t lchecktime; /* time last checked */
 	time_t check_start; /* time of start of check */
@@ -467,7 +470,8 @@ struct bootp_pkt {
 #define XML_RADIUS_SECRET	"ObjectRadiusSecret"
 #define XML_MESSAGE_ID		"ObjectMessageID"
 #define XML_UNIQUE_ID		"ObjectUniqueID"
-#define XML_OBJ_GROUP		"ObjectGroup"
+#define XML_OBJECT_GROUP	"ObjectGroup"
+#define XML_OBJECT_NOTES        "ObjectNotes"
 #define XML_OBJ_URL		"ObjectURL"
 #define XML_OBJ_URL_TEXT	"ObjectURLText"
 #define XML_OBJ_EXEC		"ObjectExecCmd"
@@ -501,6 +505,7 @@ extern char *path_savestate;
 extern char *replyto;
 extern char *downcolor, *upcolor, *recentcolor;
 extern char *statusfilename;
+extern char *cssfilename;
 extern bool quiet;
 extern time_t boottime; /* does a time() when program starts */
 extern bool debug;
@@ -564,6 +569,7 @@ extern int  parser_i_port;
 extern char *parser_numfailures;
 extern int parser_i_numfailures;
 extern char *parser_desc;
+extern char *parser_group;
 extern char *parser_spawn;
 extern char *parser_contact;
 extern char *parser_child;
@@ -579,6 +585,7 @@ extern char *parser_header;
 extern char *parser_authkey;
 extern char *parser_savestate;
 extern char *parser_statusfile;
+extern char *parser_cssfile;
 extern char *parser_pidfile;
 extern char *parser_logging;
 extern int parser_logging_fac;
@@ -619,8 +626,8 @@ void use_logging_now();
 extern bool nologconnects;
 
 /* in sysmon.c */
-void start_check_sysmon(struct monitorent *);
-void service_check_sysmon(struct monitorent *);
+void start_check_sysmon(struct monitorent *, time_t);
+void service_check_sysmon(struct monitorent *, time_t);
 void stop_check_sysmon(struct monitorent *);
 
 void start_test_dns(struct monitorent *);
@@ -628,8 +635,8 @@ void service_test_dns(struct monitorent *);
 
 void start_test_bootp(struct monitorent *);
 void service_test_bootp(struct monitorent *);
-void start_test_www(struct monitorent *);
-void service_test_www(struct monitorent *);
+void start_test_www(struct monitorent *, time_t);
+void service_test_www(struct monitorent *, time_t);
 void start_test_https(struct monitorent *);
 void service_test_https(struct monitorent *);
 
@@ -641,7 +648,7 @@ void free_struct_nei_list(struct nei_list *);
 void setup_icmp_fd();
 void handle_icmp_responses();
 void start_test_ping(struct monitorent *);
-void service_test_ping(struct monitorent *);
+void service_test_ping(struct monitorent *, struct timeval *);
 unsigned short int generate_ident();
 unsigned short in_cksum();
 
@@ -650,28 +657,33 @@ unsigned short in_cksum();
 void handle_pingv6_responses();
 void setup_icmpv6_fd();
 
+/* imap.c */
+void start_test_imap(struct monitorent *, time_t);
+void service_test_imap(struct monitorent *, time_t);
 
-void start_test_imap(struct monitorent *);
-void service_test_imap(struct monitorent *);
-void start_test_nntp(struct monitorent *);
-void service_test_nntp(struct monitorent *);
-void start_test_pop3(struct monitorent *);
-void service_test_pop3(struct monitorent *);
+/* nntp.c */
+void start_test_nntp(struct monitorent *, time_t);
+void service_test_nntp(struct monitorent *, time_t);
 
-void start_test_radius(struct monitorent *);
-void service_test_radius(struct monitorent *);
+/* pop3.c */
+void start_test_pop3(struct monitorent *, time_t);
+void service_test_pop3(struct monitorent *, time_t);
 
-void start_test_smtp(struct monitorent *);
-void service_test_smtp(struct monitorent *);
-void start_test_tcp(struct monitorent *);
-void service_test_tcp(struct monitorent *);
-void start_test_udp(struct monitorent *);
-void service_test_udp(struct monitorent *);
-void start_test_x500(struct monitorent *);
+void start_test_radius(struct monitorent *, time_t);
+void service_test_radius(struct monitorent *, time_t);
+
+void start_test_smtp(struct monitorent *, time_t);
+void service_test_smtp(struct monitorent *, time_t);
+void start_test_tcp(struct monitorent *, time_t);
+void service_test_tcp(struct monitorent *, time_t);
+void start_test_udp(struct monitorent *, time_t);
+void service_test_udp(struct monitorent *, time_t);
+void start_test_x500(struct monitorent *, time_t);
 void service_test_x500(struct monitorent *);
-void start_test_sshd(struct monitorent *);
-void start_service_sshd(struct monitorent *);
-void service_test_sshd(struct monitorent *here);
+
+void start_test_sshd(struct monitorent *, time_t);
+void service_test_sshd(struct monitorent *here, time_t);
+
 void stop_test_tcp(struct monitorent *);
 void stop_test_udp(struct monitorent *);
 void stop_test_ping(struct monitorent *);
@@ -705,13 +717,13 @@ int is_open(int);
 void set_defaults();
 void free_tree(struct all_elements_list *);
 void stop_it();
-void service_this(struct monitorent *);
+void service_this(struct monitorent *, struct timeval *, time_t);
 void print_queue(int );
 void reload_config();
 struct all_elements_list *sync_after_sighup(struct all_elements_list *, char *);
 float mydifftime(struct timeval, struct timeval);
 void blocktillready(int, int);
-void expire_dns();
+void expire_dns(time_t );
 
 /* in lib.c: */
 int str_cnt(const char *, const char);
@@ -746,16 +758,16 @@ int test_tcp(char*, int);
 int sendline(int , char *);
 
 void hard_copy(struct hostinfo *old, struct hostinfo *new);
-void dump_to_file(char *, int );
+void dump_to_file(char *, int, time_t );
 void add_line(FILE *, struct hostinfo, int, time_t);
 int data_waiting_read(int, int);
-void walk_this_way(FILE *, struct all_elements_list*, int, time_t);
+void dump_to_file_walk_this_way(FILE *, struct graph_elements*, int, time_t);
 void timeout_clients();
 void dead_client_cleanup();
 int ping(struct hostinfo *);
 char *str_difftime(time_t, time_t);
 char *str_difftime_sec(time_t, time_t);
-void page_someone(struct hostinfo*, int newstate);
+void page_someone(struct hostinfo*, int, time_t);
 struct hostinfo *parseline(int *, int*, char*, FILE*);
 int test_pop3(char*,char*,char*);
 int check_http(char*,char*, char*);
@@ -780,7 +792,7 @@ int open_sock();
 char *errtostr(int);
 char *type_to_name(int);
 char *timedata(time_t);
-void syslogmsg(char *);
+void syslogmsg(char *, time_t);
 void send_heartbeat(char *);
 
 /* auth dns check */
@@ -798,8 +810,8 @@ void service_test_snmp(struct monitorent *);
 void start_test_snmp(struct monitorent *);
 
 /* radius check */
-void start_check_radius(struct monitorent *);
-void service_check_radius(struct monitorent *);
+void start_check_radius(struct monitorent *, time_t);
+void service_check_radius(struct monitorent *, time_t);
 void stop_check_radius(struct monitorent *);
 void md5_calc (unsigned char *, unsigned char *, unsigned int);
 
