@@ -1,4 +1,4 @@
-/* $Id: lib.c,v 1.58 2004/12/16 18:41:39 jared Exp $ */
+/* $Id: lib.c,v 1.61 2006/10/06 20:54:13 jared Exp $ */
 
 #include "config.h"
 
@@ -291,6 +291,7 @@ char	*yes_no(int value)
         return "No";
 }
 
+#ifdef OLD_SOURCE
 /*
  * free parsed structure
  */
@@ -308,6 +309,7 @@ void free_parsed(struct parsed *this)
 
 	return;
 }
+#endif /* OLD_SOURCE */
 
 
 /* 
@@ -324,6 +326,7 @@ int str_cnt(const char *string, const char val)
 	return count;
 }
 
+#ifdef OLD_SOURCE
 /*
  * parse the string str, and return it in the parsed structure
  * for use -- we need to free_parsed() after we are done with it
@@ -408,6 +411,7 @@ int parse (char *str, struct parsed *parsed)
         }
 	return 0;
 }
+#endif /* OLD_SOURCE */
 
 
 char *snmp_type_to_name(int snmp_type)
@@ -670,26 +674,25 @@ void print_err (int output, const char *fmt, ...)
 	va_start (ap, fmt);
 	vsprintf (buffer, fmt, ap);
 	va_end (ap);
-	syslogmsg(buffer);
+        time (&now);
+	syslogmsg(buffer, now);
 	if (debug || output) {
-		time (&now);
 		strftime (secs, sizeof (secs), "%X", localtime (&now));
 		fprintf(stderr, "sysmond: %s %s\n", secs, buffer);
 	}
 }
 
-void	syslogmsg(char *message)
+void	syslogmsg(char *message, time_t now)
 {
 	FILE *fh;
 	struct tm *tm_val;
-	time_t now;
 	char buff[256];
 
 	if (!do_syslog)
 		return;
 	if (message == NULL)
 	{
-		syslogmsg("ATTEMPTED TO SYSLOG A NULL MESSAGE");
+		syslogmsg("ATTEMPTED TO SYSLOG A NULL MESSAGE", now);
 		return;
 	}
 
@@ -702,7 +705,6 @@ void	syslogmsg(char *message)
 		fh = fopen(log_file, "a+");
 		if (fh != NULL)
 		{
-			time(&now);
 			tm_val = localtime(&now);
 			strftime(buff, 250, "%b %d %Y-%T", tm_val);
 			fprintf(fh, "%s : %s\n", buff, message);
@@ -918,7 +920,7 @@ snprintf_convert_ulong(char *buffer, size_t buf_size, int base, char *digits,
     {
       if (base == 16 && (flags & HASH_FLAG))
       {
-          if (flags && X_UPCASE)
+          if (flags & X_UPCASE)
             {
               prefix[0] = 'x';
               prefix[1] = '0';
