@@ -1,4 +1,4 @@
-/* $Id: loadconfig.c,v 1.116 2005/09/21 01:23:27 jared Exp $ */
+/* $Id: loadconfig.c,v 1.120 2006/09/22 19:43:45 jared Exp $ */
 
 #include "config.h"
 
@@ -197,6 +197,8 @@ void free_struct_hostinfo(struct hostinfo *item_to_free)
 		FREE(item_to_free->secret);
 	if (item_to_free->lastmsgid != NULL)
 		FREE(item_to_free->lastmsgid);
+	if (item_to_free->notes != NULL)
+		FREE(item_to_free->notes);
 	if (item_to_free->unique_id != NULL)
 		FREE(item_to_free->unique_id);
 	if (item_to_free->url != NULL)
@@ -274,7 +276,7 @@ void free_tree(struct all_elements_list *freethis)
 void reload_config()
 {
 	/* log a message */
-	syslogmsg("caught SIGHUP preparing to reload config");
+	syslogmsg("caught SIGHUP preparing to reload config", time(NULL));
 	signal(SIGHUP, reload_config);
 	
 	gotsighup = TRUE; /* we need to watch for this, and when 
@@ -336,11 +338,14 @@ void	hard_copy(struct hostinfo *old, struct hostinfo *new)
 	new->totalchecked = old->totalchecked;
 	new->totaldown = old->totaldown;
 	new->last_up = old->last_up;
+	new->acked = old->acked;
 	/* copy over data related to snmp based tests */
 	new->last_snmp_resptime = old->last_snmp_resptime;
 	new->system_uptime = old->system_uptime;
 	if (old->lastmsgid != NULL)
 		new->lastmsgid = STRDUP(old->lastmsgid,"last msgid");
+	if (old->notes != NULL)
+		new->notes = STRDUP(old->notes, "old notes");
 }
 
 /*
@@ -444,6 +449,16 @@ void update_globs_from_parser()
 		statusfilename = STRDUP(parser_statusfile,"status file name");
 		html = parser_statusfile_type;
 	}
+	if (parser_cssfile != NULL)
+	{
+		if (cssfilename != NULL )
+			FREE(cssfilename);
+		if (parser_statusfile_type == 1 ) {
+			cssfilename = STRDUP(parser_cssfile, "css file name");
+		} else {
+			cssfilename = NULL;
+		}
+	}
 	if (parser_logging != NULL)
 	{
 		facility = parser_logging_fac;
@@ -621,6 +636,7 @@ struct nei_list *make_adjs_add_neighbor(struct nei_list *current_list, struct gr
 	struct nei_list *new_nei = NULL;
 	struct nei_list *nei_tmp = NULL;
 
+	/* don't add if its already there */
 	for (nei_tmp = current_list; nei_tmp != NULL; nei_tmp = nei_tmp->next)
 	{
 		if (strcmp(nei_tmp->nei_name, add->unique_name) == 0)
@@ -637,6 +653,10 @@ struct nei_list *make_adjs_add_neighbor(struct nei_list *current_list, struct gr
 
 /*
  * build adjacency list
+ *
+ * This currently results in (roughly) NumObjects^2 calls to
+ * search_dep_list which has NumObjects^5 calls to 
+ * find_object_by_name
  */
 void make_adjs(struct all_elements_list *root)
 {
@@ -681,7 +701,7 @@ void make_adjs(struct all_elements_list *root)
 		for (here2 = root; here2 != NULL; here2 = here2->next)
 		{
 			/* We needn't examine ourselves when in the 2nd depth loop */
-			if (here2 == here) /* minor optimization ? */
+			if (here2 == here)
 				continue;
 
 			/* Search here2 dep list for our (here->value->unique_name)
