@@ -1,4 +1,4 @@
-/* $Id: srvclient.c,v 1.59 2004/12/15 04:29:51 jared Exp $ */
+/* $Id: srvclient.c,v 1.68 2014/07/09 16:38:46 jared Exp $ */
 #include "config.h"
 
 extern int snmp_debug;
@@ -14,7 +14,7 @@ int deny_severity = DENYSEVERITY;
 
 void client_poll();
 
-void send_stat_start(struct clientstatus client, struct all_elements_list *head)
+void send_stat_start(struct clientstatus client, struct all_elements_list *head, int obj)
 {
 	char buff[256];
 	struct all_elements_list *here = NULL;
@@ -28,35 +28,46 @@ void send_stat_start(struct clientstatus client, struct all_elements_list *head)
 	{
 		if (here->value->data->lastcheck != 0)
 		{
-		/* down in some fashion */
+	                if (obj == 1)
+	                {
+	                        snprintf(buff, 256, "%s", here->value->unique_name);
+			} else {
+
+			/* down in some fashion */
 			snprintf(buff, 256, "%s:%d:%d:%d:%ld:%d:%ld", 
 			here->value->data->hostname, here->value->data->type, 
 			here->value->data->port, here->value->data->lastcheck,
 			here->value->data->downct, 
 			here->value->data->contacted, 
 			here->value->data->deathtime);
+			}
 
 			/* write it out the socket */
 			sendline(client.filedes, buff);
-		} 
+		}
 	}
 	return;
 }
 
-int	send_stat(struct clientstatus client)
+int	send_stat(struct clientstatus *client, char *buff)
 {
 	int retval;
-	send_stat_start(client, currenthead);
+	if (strncmp(buff, "STATO", 5) == 0)
+	{
+		send_stat_start(*client, currenthead, 1);
+	} else {
+		send_stat_start(*client, currenthead, 0);
+	}
 	if (paused)
 	{
-		retval = sendline(client.filedes, "333 Paused Currently");
+		retval = sendline(client->filedes, "333 Paused Currently");
 	} else {
-		retval = sendline(client.filedes, "333 Not currently Paused");
+		retval = sendline(client->filedes, "333 Not currently Paused");
 	}
 	if (retval == -1)
 	{
 		print_err(0, "unable to send message to client");
-		client.filedes = -1;
+		client->filedes = -1;
 		return -1;
 	}
 
@@ -92,16 +103,6 @@ int	send_conf(struct clientstatus *client)
 	return 1;
 }
 
-int	do_upd(struct clientstatus *client, char *buff)
-{
-	if (sendline(client->filedes, "444") == -1)
-	{
-		print_err(0, "unable to send message to client");
-		return -1;
-	}
-	return 0;
-}
-
 /*
  * conditionally send line to either FILE or file(can be socket)
  */
@@ -148,6 +149,20 @@ void send_object_xml(int fd, FILE *fh, struct graph_elements *obj)
 		do_send_xml(fd, fh, buffer);
 	}
 
+	/* XML_OBJ_GROUP */
+        if (obj->data->group != NULL)
+        {
+                snprintf(buffer, 1000, "<%s>%s</%s>", XML_OBJECT_GROUP, obj->data->group, XML_OBJECT_GROUP);
+                do_send_xml(fd, fh, buffer);
+        }
+
+        /* XML_OBJ_NOTES */
+        if (obj->data->notes != NULL)
+        {
+                snprintf(buffer, 1000, "<%s>%s</%s>", XML_OBJECT_NOTES, obj->data->notes, XML_OBJECT_NOTES);
+                do_send_xml(fd, fh, buffer);
+        }
+
 	if (obj->data->type == SYSM_TYPE_SNMP)
 	{
 		if (obj->data->snmp_community != NULL)
@@ -365,6 +380,98 @@ void srv_client_do_showobject(struct clientstatus *client, char *buff)
 	return;
 }
 
+/*
+ * Acklowledge an object status
+ */
+void do_ack(struct clientstatus *client, char *buff)
+{
+        char objname[128];
+        struct graph_elements *found_obj = NULL;
+
+        if (client->authlvl  <= 0)
+        {
+                sendline(client->filedes, "444 Permission Denied");
+        }
+
+        strncpy(objname, buff+4, 127);
+        objname[127] = '\0';
+        if (debug)
+                print_err(1, "will search for object :%s:", objname);
+        if (!client->xml)
+        {
+                sendline(client->filedes, "403 do mode xml first");
+                return;
+        }
+
+        found_obj = find_object_by_name(objname);
+        if (found_obj == NULL)
+        {
+                sendline(client->filedes, "403 object not found");
+                return;
+        } else {
+                found_obj->data->acked = TRUE;
+		sendline(client->filedes, "333 object updated");
+        }
+        return;
+}
+
+/*
+ * UPD <objectname> <string>
+ * like ACK but read a second line that includes a note
+ */
+int     do_upd(struct clientstatus *client, char *buff)
+{
+
+        char objname[128];
+	char *ptr = NULL;
+        struct graph_elements *found_obj = NULL;
+	int sc = 0;
+	int x;
+
+        if (client->authlvl  <= 0)
+        {
+                sendline(client->filedes, "444 Permission Denied");
+                return 0;
+        }
+
+        strncpy(objname, buff+4, 127);
+        objname[127] = '\0';
+	ptr = index(objname, ' ');
+	if (ptr != NULL)
+		*ptr = 0;
+
+	if (debug)
+		print_err(1, "do_upd/will search for object :%s:", objname);
+
+	found_obj = find_object_by_name(objname);
+	if (found_obj == NULL)
+	{
+		sendline(client->filedes, "403 object not found");
+		return 1;
+	} else {
+		for (x = 0; x < strlen(buff); x++)
+		{
+			if (buff[x] == ' ')
+				sc++;
+			if (sc == 2)
+				break;
+		}
+		if (sc == 2)
+		{
+		if (found_obj->data->notes != NULL)
+		{
+			FREE(found_obj->data->notes);
+		}
+		found_obj->data->notes = strdup(buff+x+1);
+		sendline(client->filedes, "333 object updated");
+		} else {
+			sendline(client->filedes, "403 update error");
+		}
+
+	}
+	return 0;
+}
+
 
 /*
  * MODE xml
@@ -433,16 +540,16 @@ void do_client_http(struct clientstatus *client, char *request)
 	print_err(1, "do_client_http: url = %s", url1+1);
 }
 
-void send_uptime(struct clientstatus *client)
+void send_uptime(struct clientstatus *client, time_t now_t)
 {
 	char buffer[256];
 
-	snprintf(buffer, 256, "Uptime = %s", str_difftime_sec(boottime, time(NULL)));
+	snprintf(buffer, 256, "Uptime = %s", str_difftime_sec(boottime, now_t));
 	sendline(client->filedes, buffer);
 }
 
 /* CLIENT managment code */
-void	do_service(struct clientstatus *here, char *buff)
+void	do_service(struct clientstatus *here, char *buff, time_t now_t)
 {
 	/* Local buffer */
 	char lb[256];
@@ -454,11 +561,11 @@ void	do_service(struct clientstatus *here, char *buff)
 
 	if (strncmp(buff, "STAT", 4) == 0)
 	{
-		send_stat(*here);
+		send_stat(here, buff);
 	}
 	else if (strncmp(buff, "UPTIME", 6) ==0)
 	{
-		send_uptime(here);
+		send_uptime(here, now_t);
 	}
 	else if (strncmp(buff, "NFD", 3) == 0 && (here->authlvl >1))
 	{
@@ -509,7 +616,7 @@ void	do_service(struct clientstatus *here, char *buff)
 	}
 	else if (strncmp(buff, "EXPIREDNS", 9) == 0 && (here->authlvl >1))
 	{
-		expire_dns();
+		expire_dns(now_t);
 	}
 	else if (strncmp(buff, "PRINTQ", 6) == 0 && (here->authlvl >1))
 	{
@@ -527,11 +634,11 @@ void	do_service(struct clientstatus *here, char *buff)
 	{
 		sendline(here->filedes, "333 noop");
 	}
-	else if (strncmp(buff, "SHOWOBJ", 7) == 0)
+	else if (strncmp(buff, "SHOWOBJ ", 8) == 0)
 	{
 		srv_client_do_showobject(here, buff);
 	}
-	else if (strncmp(buff, "TRACE", 5) == 0)
+	else if (strncmp(buff, "TRACE ", 6) == 0)
 	{
 		srv_client_do_trace(here, buff);
 	}
@@ -539,10 +646,14 @@ void	do_service(struct clientstatus *here, char *buff)
 	{
 		do_auth(here, buff);
 	}
-	else if (strncmp(buff, "UPD", 3) == 0)
+	else if (strncmp(buff, "UPD ", 4) == 0)
 	{
 		do_upd(here, buff);
 	}
+	else if (strncmp(buff, "ACK ", 4) == 0)
+	{
+		do_ack(here, buff);
+	}
  	else if (strncmp(buff, "GET ", 4) == 0) /* Client is a web browser */
 	{
 		do_client_http(here, buff);
@@ -759,11 +870,14 @@ void	check_for_new_clients()
 void	service_clients()
 {
 	struct clientstatus *here;
-	char throwmeout[256], buff[256];
+	char throwmeout[1024], buff[1024];
 	struct timeval tv;
 	fd_set rd,wr,except;
+	time_t now_t;
 	int maxfd;
 	int ret;
+
+	time(&now_t);
 	
 	FD_ZERO(&wr);
 	FD_ZERO(&rd);
@@ -793,8 +907,8 @@ void	service_clients()
 			{
 				getline_tcp(here->filedes, buff);
 				getline_tcp(here->filedes, throwmeout);
-				do_service(here, buff);
-				time(&here->lastactivity);
+				do_service(here, buff, now_t);
+				here->lastactivity = now_t;
 			}
 		}
 	}
