@@ -1,4 +1,4 @@
-/* $Id: syswatch.c,v 1.179 2005/10/12 02:42:23 jared Exp $ */
+/* $Id: syswatch.c,v 1.197 2014/07/09 16:29:39 jared Exp $ */
 #include "config.h"
 
 /* Normal global vars */
@@ -13,7 +13,7 @@ bool mallocdebug = FALSE; /* 0 = off, 1 = on */
 char *myname; /* myname */
 char *log_file; /* Log file used if facility = -3 */
 u_long addr=0; /* IP Addr to bind to */
-struct all_elements_list *currenthead = NULL; /* something */
+struct all_elements_list *currenthead = NULL; /* master list of objects */
 struct graph_elements *configed_root = NULL;
 struct dnscache *dnshead = NULL;
 struct clientstatus *clienthead;
@@ -73,6 +73,7 @@ char *errorsto = NULL;
 char *replyto = NULL;
 int html = -1;
 char *statusfilename = NULL;
+char *cssfilename = NULL;
 char *upcolor = NULL;
 char *downcolor = NULL;
 char *recentcolor = NULL;
@@ -132,6 +133,11 @@ void set_defaults()
 		free(statusfilename);
 		statusfilename = NULL;
 	}
+	if (cssfilename != NULL)
+	{
+		free(cssfilename);
+		cssfilename = NULL;
+	}
 	if (pmesg != NULL)
 	{
 		free(pmesg);
@@ -381,7 +387,7 @@ void save_xml_state(char *fn)
  * and other fun stuff like that, such that we are not
  * leaking anything, etc..
  */
-void stop_it()
+void stop_it(time_t now)
 {
 
 	print_err(1, "Please remain seated as your ride comes to a complete stop");
@@ -390,7 +396,7 @@ void stop_it()
 
 	/* dump current network status to a file */
 	if (html != -1)
-		dump_to_file(statusfilename, html);
+		dump_to_file(statusfilename, html, now);
 	if (path_savestate != NULL)
 		save_xml_state(path_savestate);
 
@@ -416,7 +422,7 @@ void stop_it()
 	dnsexpire = 0;
 
 	/* clear dns cache */
-	expire_dns();
+	expire_dns(now);
 
 	/* free the memory in use */
 	free_tree(currenthead);
@@ -488,7 +494,7 @@ void print_queue(int filedes)
  * This is where we actually add the object into the queue
  * for monitoring.
  */
-void queue_check(struct hostinfo *entry, char *unique_name)
+void queue_check(struct hostinfo *entry, unsigned char *unique_name)
 {
 	struct monitorent *newentry = NULL;
 
@@ -675,8 +681,13 @@ void walk_queue_checks(struct graph_elements *here, time_t now)
 /*
  * time comparison function for qsort
  */
-int q_time_cmp(const struct graph_elements *sort_a, const struct graph_elements *sort_b)
+int q_time_cmp(void **arg_a, void **arg_b)
 {
+        struct graph_elements *sort_a, *sort_b;
+
+        sort_a = *arg_a;
+        sort_b = *arg_b;
+
 	/* if they're both null, then they're equal */
 	if ((sort_a == NULL) && (sort_b == NULL))
 		return 0;
@@ -686,6 +697,9 @@ int q_time_cmp(const struct graph_elements *sort_a, const struct graph_elements
 	/* if a is null, they're not */
 	if (sort_a == NULL)
 		return -1;
+#ifdef DEBUG_QSORT
+	printf ("q_time_cmp: working on 0x%x (%s) and 0x%x\n", (unsigned int)sort_a, sort_a->unique_name, (unsigned int)sort_b);
+#endif /* DEBUG_QSORT */
 	/* if eq, return eq */
 	if ((sort_a->data == NULL) && (sort_b->data == NULL))
 		return 0;
@@ -772,6 +786,15 @@ int queue_checks_qsort_way(time_t now)
 
 		if (numele > 0)
 		{
+#ifdef DEBUG_QSORT
+		        printf("elements_to_monitor = %ld, sizeof sizeof(struct graph_elements*) = %d, alloc_size = %d\n", elements_to_monitor, sizeof(struct graph_elements*), alloc_size);
+	                printf("qsort debug:\n");
+	                for (curr = 0; curr < numele ; curr++)
+	                {
+	                        printf("\t0x%x\t%s\t%d\t%x\n", (int)queue_list[curr], queue_list[curr]->unique_name, (now-queue_list[curr]->data->next_queuetime), (int)queue_list[curr]->data);
+	                }
+#endif /* DEBUG_QSORT */
+
 			/* sort based on queue time */
 			qsort(queue_list, numele, sizeof(struct graph_elements *), q_time_cmp);
 
@@ -812,13 +835,19 @@ int queue_checks_qsort_way(time_t now)
 /*
  *
  */
-void needssleep()
+void needssleep(time_t now_t)
 {
 	fd_set rd, wr, except;
 	struct timeval local_timeout;
 	struct clientstatus *clienthere = NULL;
 	struct monitorent *here = NULL;
-	int mincalctime = 1, maxfd = 1;
+	int mincalctime = 2; /* BUG: somewhat arbitrary right now, but should work well for most folks i hope */
+	int maxfd = 1;
+	struct timeval now_timeval;
+
+        /* cache the current time */
+        gettimeofday(&now_timeval, NULL);
+
 
 	/* determine if we need sleep */
 
@@ -829,8 +858,6 @@ void needssleep()
 
 	/* Service clients */
 	client_poll();
-	handle_icmp_responses();
-
 
 	/* Watch the clients for data to be read */
 	for (clienthere = clienthead; clienthere != NULL;
@@ -888,9 +915,15 @@ void needssleep()
 		FD_SET(glob_icmpv6_fd, &rd);
 	}
 
-	/* BUG: Need a way to calculate this (really!) */
-	/* create new flags in monitorent for fd, and if
-	   we're pending read or write? */
+	/* Check: 
+	   checkent->next_queuetime for diff to now */
+        for (here=queuehead;here!= NULL;here=here->next)
+	{
+		if (here->checkent->next_queuetime > now_t)
+			if ((here->checkent->next_queuetime - now_t) < mincalctime)
+				mincalctime = here->checkent->next_queuetime - now_t;
+	}
+/*	printf("mincalctime = %d\n", mincalctime); */
 	local_timeout.tv_sec = mincalctime;
 	local_timeout.tv_usec = 0;
 
@@ -903,6 +936,7 @@ void needssleep()
 		/* and there are icmp packets */
 		if (FD_ISSET(glob_icmp_fd, &rd))
 		{
+			/* handle pending icmp responses */
 			handle_icmp_responses();
 		}
 	}
@@ -912,6 +946,7 @@ void needssleep()
 		/* and there are icmpv6 packets */
 		if (FD_ISSET(glob_icmpv6_fd, &rd))
 		{
+			/* handle pending ping icmpv6 responses */
 			handle_pingv6_responses();
 		}
 	}
@@ -938,12 +973,12 @@ void needssleep()
 			continue;
 		if (FD_ISSET(here->filedes, &rd))
 		{
-			service_this(here);
+			service_this(here, &now_timeval, now_t);
 		}
 	}
 
 	/* Expire pending dns entries */
-	expire_dns();
+	expire_dns(now_t);
 }
 
 void stop_this(struct monitorent *here)
@@ -1017,7 +1052,7 @@ void stop_this(struct monitorent *here)
        	}
 }
 
-void service_this(struct monitorent *here)
+void service_this(struct monitorent *here, struct timeval *now_timeval, time_t now_t)
 {
 	if (here == NULL)
 	{
@@ -1036,23 +1071,23 @@ void service_this(struct monitorent *here)
 		/* service a check already started */
 		switch (here->checkent->type)
 		{
-			case SYSM_TYPE_TCP: service_test_tcp(here);
+			case SYSM_TYPE_TCP: service_test_tcp(here, now_t);
 				break;
-			case SYSM_TYPE_UDP: service_test_udp(here);
+			case SYSM_TYPE_UDP: service_test_udp(here, now_t);
 				break;
-			case SYSM_TYPE_PING: service_test_ping(here);
+			case SYSM_TYPE_PING: service_test_ping(here, now_timeval);
 				break;
 #ifdef ENABLE_SNMP
 			case SYSM_TYPE_SNMP: service_test_snmp(here);
 				break;
 #endif /* ENABLE_SNMP */
-			case SYSM_TYPE_NNTP: service_test_nntp(here);
+			case SYSM_TYPE_NNTP: service_test_nntp(here, now_t);
 				break;
-			case SYSM_TYPE_SMTP: service_test_smtp(here);
+			case SYSM_TYPE_SMTP: service_test_smtp(here, now_t);
 				break;
-			case SYSM_TYPE_IMAP: service_test_imap(here);
+			case SYSM_TYPE_IMAP: service_test_imap(here, now_t);
 				break;
-			case SYSM_TYPE_POP3: service_test_pop3(here);
+			case SYSM_TYPE_POP3: service_test_pop3(here, now_t);
 				break;
 			case SYSM_TYPE_X500: service_test_x500(here);
 				break;
@@ -1064,21 +1099,21 @@ void service_this(struct monitorent *here)
 #endif /* IMPLEMENT */
 			case SYSM_TYPE_DNS: service_check_dns(here);
 				break;
-			case SYSM_TYPE_WWW: service_test_www(here);
+			case SYSM_TYPE_WWW: service_test_www(here, now_t);
 				break;
-			case SYSM_TYPE_RADIUS: service_check_radius(here);
+			case SYSM_TYPE_RADIUS: service_check_radius(here, now_t);
 				break;
 #ifdef HAVE_SSL
 			case SYSM_TYPE_HTTPS: service_test_https(here);
 				break;
 #endif /* HAVE_SSL */
-			case SYSM_TYPE_SYSM: service_check_sysmon(here);
+			case SYSM_TYPE_SYSM: service_check_sysmon(here, now_t);
 				break;
 #ifdef HAVE_IPv6
 			case SYSM_TYPE_PINGv6: service_test_pingv6(here);
 				break;
 #endif /* HAVE_IPv6 */
-			case SYSM_TYPE_SSHD: service_test_sshd(here);
+			case SYSM_TYPE_SSHD: service_test_sshd(here, now_t);
 				break;
 
 			default: print_err(1, "service_this:BUG, invalid event type");
@@ -1090,10 +1125,10 @@ void service_this(struct monitorent *here)
 		switch (here->checkent->type)
 		{
 			case SYSM_TYPE_TCP: 
-				start_test_tcp(here);
+				start_test_tcp(here, now_t);
 				break;
 			case SYSM_TYPE_UDP: 
-				start_test_udp(here);
+				start_test_udp(here, now_t);
 				break;
 			case SYSM_TYPE_PING: 
 				start_test_ping(here);
@@ -1104,15 +1139,15 @@ void service_this(struct monitorent *here)
 				break;
 #endif /* ENABLE_SNMP */
 			case SYSM_TYPE_NNTP: 
-				start_test_nntp(here);
+				start_test_nntp(here, now_t);
 				break;
-			case SYSM_TYPE_SMTP: start_test_smtp(here);
+			case SYSM_TYPE_SMTP: start_test_smtp(here, now_t);
 				break;
-			case SYSM_TYPE_IMAP: start_test_imap(here);
+			case SYSM_TYPE_IMAP: start_test_imap(here, now_t);
 				break;
-			case SYSM_TYPE_POP3: start_test_pop3(here);
+			case SYSM_TYPE_POP3: start_test_pop3(here, now_t);
 				break;
-			case SYSM_TYPE_X500: start_test_x500(here);
+			case SYSM_TYPE_X500: start_test_x500(here, now_t);
 				break;
 #ifdef IMPLEMENT
 			case SYSM_TYPE_POP2: start_test_pop2(here);
@@ -1122,21 +1157,21 @@ void service_this(struct monitorent *here)
 #endif /* IMPLEMENT */
 			case SYSM_TYPE_DNS: start_check_dns(here);
 				break;
-			case SYSM_TYPE_WWW: start_test_www(here);
+			case SYSM_TYPE_WWW: start_test_www(here, now_t);
 				break;
-			case SYSM_TYPE_RADIUS: start_check_radius(here);
+			case SYSM_TYPE_RADIUS: start_check_radius(here, now_t);
 				break;
 #ifdef HAVE_SSL
 			case SYSM_TYPE_HTTPS: start_test_https(here);
 				break;
 #endif /* HAVE_SSL */
-			case SYSM_TYPE_SYSM: start_check_sysmon(here);
+			case SYSM_TYPE_SYSM: start_check_sysmon(here, now_t);
 				break;
 #ifdef HAVE_IPv6
 			case SYSM_TYPE_PINGv6: start_test_pingv6(here);
 				break;
 #endif /* HAVE_IPv6 */
-			case SYSM_TYPE_SSHD: start_test_sshd(here);
+			case SYSM_TYPE_SSHD: start_test_sshd(here, now_t);
 				break;
 
 			default: print_err(1, "service_this:BUG, invalid event type");
@@ -1189,10 +1224,10 @@ void handle_retval(struct monitorent *handle_this, time_t now)
 		{
 			/* page */
 			if (debug) print_err(1, "handle_retval:calling page_someone");
-			page_someone(handle_this->checkent, SYSM_CONTACT_UP);
+			page_someone(handle_this->checkent, SYSM_CONTACT_UP, now);
 		}
 
-		time(&handle_this->checkent->last_up);
+		handle_this->checkent->last_up = now;
 
 		/* reset contacted */
 		handle_this->checkent->contacted = FALSE; /* init it */
@@ -1219,7 +1254,7 @@ void handle_retval(struct monitorent *handle_this, time_t now)
 			(handle_this->checkent->contact_when & SYSM_CONTACT_DOWN))
 		{
 			/* page someone */
-			page_someone(handle_this->checkent, SYSM_CONTACT_DOWN);
+			page_someone(handle_this->checkent, SYSM_CONTACT_DOWN, now);
 		}
 
 	} /* endif some other retval check thing */
@@ -1231,7 +1266,7 @@ void handle_retval(struct monitorent *handle_this, time_t now)
 		handle_this->checkent->downct = 1;
 		handle_this->checkent->upct = 0;
 		handle_this->checkent->lastcheck = handle_this->retval;
-		time(&handle_this->checkent->deathtime);
+		handle_this->checkent->deathtime = now;
 		handle_this->checkent->totaldown++;
 
 		if (debug) print_err(1, "handle_retval downct = %d, max %d", handle_this->checkent->downct, handle_this->checkent->max_down);
@@ -1244,7 +1279,7 @@ void handle_retval(struct monitorent *handle_this, time_t now)
 			(handle_this->checkent->contact_when & SYSM_CONTACT_DOWN))
 		{
 			/* page someone */
-			page_someone(handle_this->checkent, SYSM_CONTACT_DOWN);
+			page_someone(handle_this->checkent, SYSM_CONTACT_DOWN, now);
 		}
 
 	} /*endif handle_this->retval!=handle_this->checkent->lastcheck*/
@@ -1261,7 +1296,7 @@ void handle_retval(struct monitorent *handle_this, time_t now)
 			(handle_this->checkent->contact_when == SYSM_CONTACT_UP))
 		{
 			/* page someone */
-			page_someone(handle_this->checkent, SYSM_CONTACT_UP);
+			page_someone(handle_this->checkent, SYSM_CONTACT_UP, now);
 		}
 	}
 
@@ -1273,17 +1308,20 @@ void handle_retval(struct monitorent *handle_this, time_t now)
 /*
  *
  */
-void service_checks()
+void service_checks(time_t now)
 {
 	struct monitorent *here, *last, *freeit;
-	time_t now;
+        struct timeval now_timeval;
+
 
 	if (queuehead == NULL)
 	{
 		return;
 	}
 
-	time(&now); /* get current time */
+        /* cache the current time */
+        gettimeofday(&now_timeval, NULL);
+
 
 	/* setup checks that need to be, service current checks */
 	last = NULL;
@@ -1294,7 +1332,7 @@ void service_checks()
 		/* if no return value */
 		if (here->retval == -1)
 		{
-			service_this(here); /* service the check */
+			service_this(here, &now_timeval, now); /* service the check */
 			last = here;
 		} else {
 			/* Handle the return value */
@@ -1507,9 +1545,9 @@ do_watch(char *cmdname, int listenport, char *myhostname)
 {
 	struct all_elements_list *hupdata = NULL;
 	time_t now_t, last_t;
+	time_t last_periodic_t = 0;
 	last_t = 0;
 
-
 #ifdef HAVE_LIBPTHREAD
 	pthread_mutex_init(&Sysmon_Giant, NULL);
 #endif /* HAVE_LIBPTHREAD */
@@ -1528,7 +1566,7 @@ do_watch(char *cmdname, int listenport, char *myhostname)
 	while (1)
 	{
 		time(&now_t);
-		if (currenthead != NULL) 
+		if (currenthead != NULL && (numqueued <= maxqueued)) 
 		/* something in the tree to monitor */
 		{
 #ifndef QSORT_WAY
@@ -1544,30 +1582,37 @@ do_watch(char *cmdname, int listenport, char *myhostname)
 			last_t = now_t;
 		}
 
-		service_checks(); /* do checks that are ready for us */
+		service_checks(now_t); /* do checks that are ready for us */
 
-		needssleep();     /* if we need a sleep, do it here */
+		needssleep(now_t);     /* if we need a sleep, do it here */
 
-		do_tree_periodic(now_t); /* Do periodic checks/tests */
+		/* only do this once every 60 seconds */
+		if ((now_t - last_periodic_t) > 60)
+		{
+			do_tree_periodic(now_t); /* Do periodic checks/tests */
+			last_periodic_t = now_t;
+		}
 
 		/* dump current network status to a file */
 		if ((html != -1) && (statuschanged))
 		{
-			dump_to_file(statusfilename, html);
+			dump_to_file(statusfilename, html, now_t);
 			statuschanged = FALSE;
 		}
 
-		while (paused)
+		/* */
+		while (paused && (!gotsighup))
 		{
-			service_checks();
+			time(&now_t);
+			service_checks(now_t);
 			wakeup_checks();
 			do_tree_periodic(now_t);
-			needssleep();
+			needssleep(now_t);
 		}
 
 		if (stop_daemon)
 		{
-			stop_it();
+			stop_it(now_t);
 		}
 
 		if (gotsighup)
@@ -1583,6 +1628,9 @@ do_watch(char *cmdname, int listenport, char *myhostname)
 			hupdata = sync_after_sighup(currenthead, configfile);
 
 			currenthead = hupdata;
+			/* Must do this to avoid qsort having out of bounds issues
+			 * fix from corne.cornelius/at/gmail.com */
+			update_count();
 
 			if (debug)
 			{
@@ -1616,7 +1664,13 @@ void periodic_rewalk(struct all_elements_list *here, time_t right_now, int down)
 				type_to_name(loc->value->data->type), 
 				loc->value->data->port, 
 				(right_now - loc->value->data->lchecktime));
+#ifdef HAVE_LIBPTHREAD
+			pthread_mutex_lock(&Sysmon_Giant);
+#endif /* HAVE_LIBPTHREAD */
 			loc->value->data->warnlog = 1;
+#ifdef HAVE_LIBPTHREAD
+			pthread_mutex_unlock(&Sysmon_Giant);
+#endif /* HAVE_LIBPTHREAD */
 		}
 		}
 		if (loc->value->data->lastcheck == SYSM_NODNS)
@@ -1625,7 +1679,14 @@ void periodic_rewalk(struct all_elements_list *here, time_t right_now, int down)
 			if (my_gethostbyname(loc->value->data->hostname, -1) 
 				!= NULL)
 			{
+#ifdef HAVE_LIBPTHREAD
+				pthread_mutex_lock(&Sysmon_Giant);
+#endif /* HAVE_LIBPTHREAD */
 				loc->value->data->lastcheck = 0;
+#ifdef HAVE_LIBPTHREAD
+				pthread_mutex_unlock(&Sysmon_Giant);
+#endif /* HAVE_LIBPTHREAD */
+
 			}
 		}
 		}
@@ -1659,7 +1720,7 @@ void walk_periodic_page_checks(struct graph_elements *here, time_t now)
 	if (((now - here->data->lastcontacted) > (pageinterval*60)) &&
 		(here->data->contacted))
 	{
-		page_someone(here->data, SYSM_CONTACT_DOWN);
+		page_someone(here->data, SYSM_CONTACT_DOWN, now);
 	}
 
 	/* BUG: walk_periodic_page_checks: are we doing this right? */
@@ -1769,7 +1830,7 @@ int main(int argc, char **argv)
 	boottime = time(NULL);
 
 	myname = argv[0];
-	time(&dnslog_last_log);
+	dnslog_last_log = boottime;
 
 	currenthead = NULL; /* initalize it */
 
